import { IProduct } from '@/interfaces/IProduct.interface';
import { ProductDto } from '@/dto/ProductDto';
import * as fs from 'fs';
import path from 'path';
import { randomUUID } from 'crypto';

const filePath = path.join(__dirname, '../data/products.json');

export class ProductService {
  private products: IProduct[] = [];

  constructor() {
    this.init();
  }

  private init() {
    try {
      const fileContents = fs.readFileSync(filePath);
      this.products = JSON.parse(fileContents.toString());
    } catch (e) {
      this.products = [];
    }
  }

  private save() {
    fs.writeFileSync(filePath, JSON.stringify(this.products, null, 2));
  }

  public getProducts(): IProduct[] {
    return this.products;
  }

  public getProduct(id: string): IProduct {
    const singleProduct = this.products.find((product) => product.id === id);
    if (singleProduct) return singleProduct;
    else throw new Error('Invalid id');
  }

  public createProduct(productDto: ProductDto): IProduct {
    const product = {
      id: randomUUID(),
      ...productDto,
    };

    this.products.push(product);
    this.save();
    return product;
  }
}
